--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

-- TODO : Replace all Crayon:White with colors in layout?


local deformat = AceLibrary("Deformat-2.0")
local crayon = AceLibrary("Crayon-2.0")
local tourist = AceLibrary("Tourist-2.0")
local quixote = AceLibrary("Quixote-1.0")

function nQuestLog:Format_QuestZone(zone)
	local level = ""

	local low, high = tourist:GetLevel(zone)
	if (self.db.profile.showZoneLevel and low > 0 and high > 0) then
		level = self:Colorize(" " .. low .. "-" .. high,"zoneLevel",low / high,zone)
	end
	
	zone = self:Colorize(zone,"zoneHeader",low / high,zone)
	
	return zone .. level
end


function nQuestLog:Format_QuestTitle(title,level,tag,got,needed,complete)
	local newtitle = "[" .. level .. tag .. "]" .. title
	
	--Color quest name
	local newtitle = self:Colorize("[" .. level .. tag .. "]" .. title,"questHeader",nil,nil,level)	

	--Color quest progress
	local progress = ""
	if (complete == -1) then
		-- Quest failed
		progress = self:Colorize(" (failed)","questFailed")
	elseif (complete == 1) then
		-- Quest completed
		progress = self:Colorize(" (done)","questDone")
	else
		if (needed == 0) then
			progress = self:Colorize(" (goto)","questGoto")
		elseif self.db.profile.showQuestProgress then
			progress = self:Colorize(" " .. got .. "/" .. needed,"questProgress",got/needed)
		end
	end

	return "  " .. newtitle .. progress
end



function nQuestLog:UpdateSmartObjectiveTracking()
	-- This function should be called before scanning through quest objectives
	if (not self.db.profile.smartObjectiveTracking) then
		return
	end

	-- self:Print("Doing smart quest tracking")

	if (self.db.profile.smartObjectiveTracking_MaxAge ~= 0) then
		local minTime = GetTime() - (self.db.profile.smartObjectiveTracking_MaxAge * 60)
		-- Remove quests that are too old
		for name,time in pairs(self.ObjectivesDB) do
			if time ~= true and time ~= false then
				if minTime >  time then
					-- self.ObjectivesDB[title] is time (ie not manual) and is too old, remove it
					self:Print("Quest objective " .. name .. " too old, hiding.")
					self.ObjectivesDB[name] = nil
				end
			end
		end
	end
	
	local function removeOldest()
		-- Function to always remove the oldest quest:
		local oldestName
		local oldestTime = GetTime()
		local questsTracked = 0
		
		for name,time in pairs(self.ObjectivesDB) do
			if time ~= true and time ~= false then
				if oldestTime > time then
					oldestTime = time
					oldestName = name
				end
				questsTracked = questsTracked + 1
			end
		end
		
		if (questsTracked > self.db.profile.smartObjectiveTracking_Max) then
			self:Print("Too many quest objectives tracked, removing monitoring for " .. oldestName)
			self.ObjectivesDB[oldestName] = nil
			questsTracked = questsTracked - 1
			if (questsTracked > self.db.profile.smartObjectiveTracking_Max) then
				removeOldest()
			end
		end
	end
	
	if (self.db.profile.smartObjectiveTracking_Max > 0) then
		removeOldest()
	end
end

function nQuestLog:ShouldShowObjective(title, got, needed)
	-- Returns true if we should show a specific objective (title = quest title, not objective title)
	
	-- Todo, should not show objectives on failed quests.
	-- TODO, when scanning and removing old quests we should take zone into consideration, to hide out of zone quests before other quests?
	-- Can also take level into consideration, and let it weight a bit atleast
	-- prioritize stuff where we have 0/total over removing things where we have some objectives
	
	-- Function to check if quest objectives should be shown.
	if (self.ObjectivesDB[title] == true or self.ObjectivesDB[title] == false) then
		-- self.ObjectivesDB[title] overrides every setting
		return self.ObjectivesDB[title]
	end

	-- Hide completed objectives even with smart quest tracking
	if (self.db.profile.hideCompletedObjectives and got == needed) then
		-- Hide completed
		return false
	end
	
	-- Smart quest tracking:
	if (self.db.profile.smartObjectiveTracking) then
		-- See ScanSmartObjectiveTracking(), if this isn't nil and smart objective tracking is on we should track this quest
		if (self.ObjectivesDB[title] ~= nil) then
			return true
		end
		return false
	end
	
	if (self.db.profile.hideAllObjectives) then
		-- Hide everything, simple
		return false
	end
	
	return true
end

function nQuestLog:Format_QuestObjectives(title,objectives,objective,id,force)
	-- Large function to format quest objectives
	local newobjectives = ""
	local totalGot, totalNeeded = 0,0
	
	local function Format_QuestObjective(desc, got, needed)
		-- Function to format a specific quest objective

		local desc = self:Colorize(desc,"objectiveHeader",got / needed)
		local progress = self:Colorize(got .. "/" .. needed,"objectiveProgress",got / needed)
		return "\n   - " .. desc .. " " .. progress
	end


	-- Loop through quest objectives and add them all
	if (objectives > 0) then
		for i=1,objectives do
			local desc, type, got, needed, done = quixote:GetQuestStatusById(id, i)
			
			if (type == "reputation") then needed = 1	got = 0	if (done) then got = 1 end end
			totalGot = totalGot + got
			totalNeeded = totalNeeded + needed
			
			if (force or self:ShouldShowObjective(title,got,needed)) then
				newobjectives = newobjectives .. Format_QuestObjective(desc, got, needed)
			end
		end
	else
		if (not force and self:ShouldShowObjective(title)) then
			newobjectives = "\n   - " .. crayon:Copper(objective) -- TODO make this a configurable color
		end
	end
	
	if (newobjectives == "" and self.ObjectivesDB[title] and (self.ObjectivesDB[title] ~= true or self.ObjectivesDB[title] ~= false)) then
		-- Clear the timestamp, since the quest objectives are no longer being tracked.
		self.ObjectivesDB[title] = nil
	end
	
	return newobjectives,totalGot,totalNeeded
end

